from mat import Mat
import math

## Task 1
def identity(labels = {'x','y','u'}):
    '''
    In case you have never seen this notation for a parameter before,
    the way it works is that identity() now defaults to having labels 
    equal to {'x','y','u'}.  So you should write your procedure as if 
    it were defined 'def identity(labels):'.  However, if you want the labels of 
    your identity matrix to be {'x','y','u'}, you can just call 
    identity().  Additionally, if you want {'r','g','b'}, or another set, to be the
    labels of your matrix, you can call identity({'r','g','b'}).  
    '''
    #pass
    return Mat((labels,labels),{(x,y):1 for x in labels for y in labels if x == y})

## Task 2
def translation(x,y):
    '''
    Input:  An x and y value by which to translate an image.
    Output:  Corresponding 3x3 translation matrix.
    '''
    #pass
    M = Mat(({'x','y','u'},{'x','y','u'}),{})
    M['u','u'] = 1
    M['x','u'] = x
    M['y','u'] = y
    M['x','x'] = 1
    M['y','y'] = 1
    return M

## Task 3
def scale(a, b):
    '''
    Input:  Scaling parameters for the x and y direction.
    Output:  Corresponding 3x3 scaling matrix.
    '''
    #pass
    M = Mat(({'x','y','u'},{'x','y','u'}),{})
    M['x','x'] = a
    M['y','y'] = b
    M['u','u'] = 1
    return M

## Task 4
def rotation(angle):
    '''
    Input:  An angle in radians to rotate an image.
    Output:  Corresponding 3x3 rotation matrix.
    Note that the math module is imported.
    '''
    #pass
    M = Mat(({'x','y','u'},{'x','y','u'}),{})
    M['x','x'] = math.cos(angle)
    M['y','y'] = math.cos(angle)
    M['x','y'] = -math.sin(angle)
    M['y','x'] = math.sin(angle)
    M['u','u'] = 1
    return M

## Task 5
def rotate_about(x,y,angle):
    '''
    Input:  An x and y coordinate to rotate about, and an angle
    in radians to rotate about.
    Output:  Corresponding 3x3 rotation matrix.
    It might be helpful to use procedures you already wrote.
    '''
    #pass
    Mt = translation(-x,-y)
    Mr = rotation(angle)
    Mt2 = translation(x,y)
    #M = Mt * Mr * Mt2
    M = Mt2 * Mr * Mt
    return M

## Task 6
def reflect_y():
    '''
    Input:  None.
    Output:  3x3 Y-reflection matrix.
    '''
    #pass
    return scale(-1,1)

## Task 7
def reflect_x():
    '''
    Inpute:  None.
    Output:  3x3 X-reflection matrix.
    '''
    #pass
    return scale(1,-1)
    
## Task 8    
def scale_color(scale_r,scale_g,scale_b):
    '''
    Input:  3 scaling parameters for the colors of the image.
    Output:  Corresponding 3x3 color scaling matrix.
    '''
    #pass
    I = identity({'r','g','b'})
    I['r','r'] = scale_r
    I['g','g'] = scale_g
    I['b','b'] = scale_b
    return I

## Task 9
def grayscale():
    '''
    Input: None
    Output: 3x3 greyscale matrix.
    '''
    #pass   
    #GS = Mat(({'x','y','u'},{'r','g','b'}),{})
    GS = Mat(({'r','g','b'},{'r','g','b'}),{})
    GS['r','r'] = 77/256
    GS['g','r'] = 77/256
    GS['b','r'] = 77/256
    GS['r','g'] = 151/256
    GS['g','g'] = 151/256
    GS['b','g']	= 151/256
    GS['r','b'] = 28/256
    GS['g','b'] = 28/256
    GS['b','b'] = 28/256
    return GS

## Task 10
def reflect_about(p1,p2):
    '''
    Input: 2 points that define a line to reflect about.
    Output:  Corresponding 3x3 reflect about matrix.
    '''
    #pass
    x1, y1, x2, y2 = p1[0], p1[1], p2[0], p2[1]
    theta = math.atan2(y2 - y1, x2 - x1)
    return translation(x2,y2)*rotation(theta)*reflect_x()*rotation(-theta)*translation(-x2,-y2)



